require 'test_helper'

class TagTest < ActiveSupport::TestCase
  should have_many :taggings
  should validate_presence_of :name

  test 'an user can tag a learning object' do
    john = users(:john)
    lo = learning_objects(:one)
    assert john.tag(lo, with: ['Tag'])

    # check lo_one tags
    assert_count 2, lo.tags
    assert_count 2, john.taggings
  end

  test 'an user can untag a learning object' do
    john = users(:john)
    lo = learning_objects(:one)
    assert john.untag(lo, 'Tag')

    # check lo_one tags
    assert_count 1, lo.tags
    assert_count 1, john.taggings
  end

  test 'an user can tag a collection' do
    john = users(:john)
    collection = collections(:ufpr)
    assert john.tag(collection, with: ['Tag'])

    # check ufpr tags
    assert_count 3, collection.tags
    assert_count 2, john.taggings
  end

  test 'an user can untag a collection' do
    john = users(:john)
    collection = collections(:ufpr)
    assert john.untag(collection, 'Tag')

    # check ufpr tags
    assert_count 2, collection.tags
    assert_count 1, john.taggings
  end


end
